/*  OpenTally: Open-source election vote counting
 *  Copyright © 2021–2023 Lee Yingtong Li (RunasSudo)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

use crate::utils;

use opentally::numbers::Rational;
use opentally::stv;
use opentally::ties::TieStrategy;

#[test]
fn vec_nmet22_rational() {
	let stv_opts = stv::STVOptionsBuilder::default()
		.round_votes(Some(0))
		.round_quota(Some(0))
		.quota_criterion(stv::QuotaCriterion::GreaterOrEqual)
		.ties(vec![TieStrategy::Backwards])
		.surplus(stv::SurplusMethod::UIG)
		.surplus_order(stv::SurplusOrder::ByOrder)
		.exclusion(stv::ExclusionMethod::FirstPreferencesThenByValue)
		.early_bulk_elect(false)
		.build().unwrap();
	
	assert_eq!(stv_opts.describe::<Rational>(), "--round-votes 0 --round-quota 0 --quota-criterion geq --ties backwards --surplus uig --surplus-order by_order --exclusion first_prefs_then_by_value --no-early-bulk-elect");
	
	utils::read_validate_election::<Rational>("tests/data/SE2022_NMET.csv", "tests/data/SE2022_NMET.blt", stv_opts, None, &["lbf", "exhausted"]);
}
